/***************************************************************************
*   Copyright (C) 2004-2007 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qregexp.h>
#include <qtabwidget.h>
#include <qdom.h>
#include <qfile.h>
#include <qtextedit.h>
#include <qapplication.h>
#include <qhttp.h>
#include <qlayout.h>
#include <qurl.h>
#include <qlabel.h>
#include <qbuffer.h>
#include <qcstring.h>
#include <qlistview.h>

#include <klocale.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kdialog.h>
#include <kurl.h>
#include <kdebug.h>
#include <kglobalsettings.h>

#include <entry.h>
#include "webquerysru.h"

namespace KBibTeX
{

    WebQuerySRU::WebQuerySRU( QWidget* parent, const char* name ) : WebQuery()
    {
        wizard = new SRU::SRUWizard( title(), parent, name );
    }


    WebQuerySRU::~WebQuerySRU()
    {
        // TODO
    }

    BibTeX::File* WebQuerySRU::query( )
    {
        if ( wizard->exec() == QDialog::Accepted )
            return wizard->result();
        else
            return NULL;
    }

    QString WebQuerySRU::title()
    {
        return i18n( "SRU" );
    }

    int WebQuerySRU::uniqueId()
    {
        return 0x0783a5bc;
    }

    SRU::SRUWizard::SRUWizard( const QString &caption, QWidget* parent, const char* name ) : KDialogBase( parent, name, TRUE, "Import", Ok | Cancel, Ok, TRUE ), bibtexFile( NULL )
    {
        setCaption( caption );
        setupGUI();
    }

    SRU::SRUWizard::~SRUWizard()
    {
        // nothing
    }

    BibTeX::File *SRU::SRUWizard::result()
    {
        return bibtexFile;
    }

    int SRU::SRUWizard::exec()
    {
        int result = KDialogBase::exec();

        return result;
    }

    void SRU::SRUWizard::accept()
    {
        bibtexFile = new BibTeX::File();
        for ( QListViewItemIterator it( m_listViewResults, QListViewItemIterator::Selected ); it.current();++it )
        {
            ResultsListViewItem *item = dynamic_cast<ResultsListViewItem*>( *it );
            if ( item != NULL )
                bibtexFile->appendElement( BibTeX::File::cloneElement( item->entry() ) );
        }

        KDialogBase::accept();
    }

    void SRU::SRUWizard::startSearch()
    {
        setEnabled( FALSE );
        QApplication::setOverrideCursor( Qt::waitCursor );
        m_listViewResults->clear();

        QString urlBase = "%1?operation=searchRetrieve&version=1.1&query=%3&maximumRecords=%2&recordPacking=xml&recordSchema=dc";
        QString server = m_comboBoxServer->currentText();
        QString query = m_lineEditQuery->text();
        int hits = m_spinBoxMaxHits->value();
        QUrl url( urlBase.arg( server ).arg( hits ).arg( query ) );

        http = new QHttp( this );
        QBuffer *buffer = new QBuffer( incomingData );
        connect( http, SIGNAL( stateChanged( int ) ), this, SLOT( httpStateChanged( int ) ) );
        connect( http, SIGNAL( done( bool ) ), this, SLOT( searchRetrieveDone( bool ) ) );
        int port = url.port();
        if ( port == -1 ) port = 80;
        http->setHost( url.host(), port );
        http->get( url.path() + "?" + url.query(), buffer );
    }

    void SRU::SRUWizard::searchRetrieveDone( bool error )
    {
        kdDebug() << "SRU::SRUWizard::searchRetrieveDone( " << error << " )" << endl;

        disconnect( http, SIGNAL( done( bool ) ), this, SLOT( searchRetrieveDone( bool ) ) );

        if ( error )
        {
            setEnabled( TRUE );
            QApplication::restoreOverrideCursor();
            KMessageBox::error( this, QString( i18n( "Could not query database at \"%2\":\n%1" ) ).arg( http->errorString() ).arg( m_comboBoxServer->currentText() ) );
            return ;
        }

        QBuffer *buffer = new QBuffer( incomingData );

        buffer->open( IO_ReadOnly );
        QTextStream ts( buffer );
        ts.setEncoding( QTextStream::UnicodeUTF8 );
        QFile output( "/tmp/receiveddata.txt" );
        output.open( IO_WriteOnly );
        QTextStream outputts( &output );
        outputts << ts.read();
        output.close();
        buffer->close();

        buffer->open( IO_ReadOnly );
        m_sourceWidget->setText( ts.read() );
        buffer->close();

        buffer->open( IO_ReadOnly );
        QDomDocument doc;
        doc.setContent( buffer );
        ResultParser resultParser( m_listViewResults, doc.documentElement() );
        buffer->close();

        setEnabled( TRUE );
        QApplication::restoreOverrideCursor();
        m_listViewResults->setFocus();
    }

    void SRU::SRUWizard::listViewResultsSelectionChanged( )
    {
        QListViewItemIterator it( m_listViewResults, QListViewItemIterator::Selected );

        enableButtonOK( it.current() != NULL );
    }

    void SRU::SRUWizard::httpStateChanged( int state )
    {
        switch ( state )
        {
        case QHttp::Unconnected:
            kdDebug() << "HTTP state: QHttp::Unconnected" << endl;
            break;
        case QHttp::HostLookup:
            kdDebug() << "HTTP state: QHttp::HostLookup" << endl;
            break;
        case QHttp::Connecting:
            kdDebug() << "HTTP state: QHttp::Connecting" << endl;
            break;
        case QHttp::Sending:
            kdDebug() << "HTTP state: QHttp::Sending" << endl;
            break;
        case QHttp::Reading:
            kdDebug() << "HTTP state: QHttp::Reading" << endl;
            break;
        case QHttp::Connected:
            kdDebug() << "HTTP state: QHttp::Connected" << endl;
            break;
        case QHttp::Closing:
            kdDebug() << "HTTP state: QHttp::Closing" << endl;
            break;
        default:
            kdDebug() << "unknown state: " << state << endl;
        }
    }

    void SRU::SRUWizard::setupGUI()
    {
        QTabWidget * tabWidget = new QTabWidget( this );

        QWidget * container = new QWidget( tabWidget );
        tabWidget->addTab( container, i18n( "Search" ) );
        QGridLayout *gridLayout = new QGridLayout( container, 2, 7, KDialog::marginHint(), KDialog::spacingHint() );
        gridLayout->setColStretch( 1, 3 );
        gridLayout->setColStretch( 3, 10 );
        gridLayout->setRowStretch( 1, 10 );
        QLabel *label = new QLabel( i18n( "Ser&ver:" ), container );
        gridLayout->addWidget( label, 0, 0 );
        m_comboBoxServer = new QComboBox( TRUE, container );
        gridLayout->addWidget( m_comboBoxServer, 0, 1 );
        label->setBuddy( m_comboBoxServer );

        label = new QLabel( i18n( "&Query:" ), container );
        gridLayout->addWidget( label, 0, 2 );
        m_lineEditQuery = new QLineEdit( container );
        gridLayout->addWidget( m_lineEditQuery, 0, 3 );
        label->setBuddy( m_lineEditQuery );

        label = new QLabel( i18n( "&Number of results:" ), container );
        gridLayout->addWidget( label, 0, 4 );
        m_spinBoxMaxHits = new QSpinBox( 5, 5000, 1, container );
        m_spinBoxMaxHits->setValue( 3 );
        gridLayout->addWidget( m_spinBoxMaxHits, 0, 5 );
        label->setBuddy( m_spinBoxMaxHits );

        m_pushButtonSearch = new QPushButton( i18n( "&Search" ), container );
        gridLayout->addWidget( m_pushButtonSearch, 0, 6 );
        m_pushButtonSearch->setIconSet( QIconSet( SmallIcon( "find" ) ) );

        m_listViewResults = new QListView( container );
        m_listViewResults->addColumn( i18n( "Year" ), 64 );
        m_listViewResults->addColumn( i18n( "Author" ), 128 );
        m_listViewResults->addColumn( i18n( "Title" ), 512 );
        m_listViewResults->setAllColumnsShowFocus( TRUE );
        m_listViewResults->setSelectionMode( QListView::Extended );
        gridLayout->addMultiCellWidget( m_listViewResults, 1, 1, 0, 6 );

        m_lineEditQuery->setFocus();
        setMainWidget( tabWidget );
        setButtonOK( KGuiItem( i18n( "&Import" ), "import", i18n( "Import selected items" ) ) );
        enableButtonOK( FALSE );
        setPlainCaption( i18n( "Import from SRU" ) );
        // TODO

        connect( m_pushButtonSearch, SIGNAL( clicked() ), this, SLOT( startSearch() ) );
        connect( m_listViewResults, SIGNAL( selectionChanged( ) ), this, SLOT( listViewResultsSelectionChanged( ) ) );
        connect( m_listViewResults, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( listViewResultsSelectionChanged( ) ) );
        connect( m_lineEditQuery, SIGNAL( returnPressed() ), this, SLOT( startSearch() ) );
        connect( m_comboBoxServer->lineEdit(), SIGNAL( returnPressed() ), this, SLOT( startSearch() ) );

        m_comboBoxServer->insertItem( "http://z3950.loc.gov:7090/voyager" );
        m_comboBoxServer->insertItem( "http://copac.ac.uk:3000/copac" );
        m_comboBoxServer->insertItem( "http://krait.kb.nl/cgi-zoek/sru.pl" );
        m_comboBoxServer->insertItem( "http://herbie.bl.uk:9080/cgi-bin/blils.cgi" );
        m_comboBoxServer->setCurrentItem( 0 );
        m_lineEditQuery->setText( "dinosaur" );

        container = new QWidget( tabWidget );
        tabWidget->addTab( container, i18n( "Source" ) );
        gridLayout = new QGridLayout( container, 1, 1, KDialog::marginHint(), KDialog::spacingHint() );
        m_sourceWidget = new QTextEdit( container );
        gridLayout->addWidget( m_sourceWidget, 0, 0 );
        m_sourceWidget->setReadOnly( TRUE );
        m_sourceWidget->setFont( KGlobalSettings::fixedFont() );

    }

    SRU::ResultParser::ResultParser( QListView *listView, const QDomElement& rootElement )
    {
        bool badFormat = false;
        kdDebug() << " rootElement.tagName() = " << rootElement.tagName() << endl;

        if ( rootElement.tagName().endsWith( "searchRetrieveResponse" ) )
        {
            for ( QDomNode n = rootElement.firstChild(); !badFormat && !n.isNull(); n = n.nextSibling() )
            {
                QDomElement e = n.toElement();
                if ( !e.isNull() && e.tagName().endsWith( "records" ) )
                {
                    for ( QDomNode n2 = e.firstChild(); !badFormat && !n2.isNull(); n2 = n2.nextSibling() )
                    {
                        QDomElement e2 = n2.toElement();
                        if ( !e2.isNull() && e2.tagName().endsWith( "record" ) )
                        {
                            BibTeX::Entry * entry = parseRecord( e2 );
                            if ( entry != NULL )
                                new ResultsListViewItem( listView, entry );
                            else
                                badFormat = true;
                        }
                    }
                }
            }
        }
        else
            kdDebug() << "Unknown root element: " << rootElement.tagName() << endl;

        if ( badFormat )
        {
            listView->clear();
            KMessageBox::error( listView, i18n( "The server returned the search results in a schema, that is not supported by KBibTeX." ), i18n( "Unsupported content schema" ) );
        }
    }

    SRU::ResultParser::~ResultParser()
    {
        // nothing
    }

    BibTeX::Entry * SRU::ResultParser::parseRecord( const QDomElement & element )
    {
        BibTeX::Entry * entry = NULL;

        QString schema;
        QString packing;
        for ( QDomNode n = element.firstChild();( schema.isEmpty() || packing.isEmpty() ) && !n.isNull(); n = n.nextSibling() )
        {
            QDomElement e = n.toElement();
            if ( !e.isNull() )
            {
                qDebug( " e.tagName()=%s", e.tagName().latin1() );
                if ( e.tagName().endsWith( "recordSchema" ) )
                {
                    schema = e.text();
                    if ( schema == "info:srw/schema/1/dc-v1.1" )
                        schema = "dc";
                }
                else if ( e.tagName().endsWith( "recordPacking" ) )
                    packing = e.text();
            }
        }

        for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
        {
            QDomElement e = n.toElement();
            if ( !e.isNull() && e.tagName().endsWith( "recordData" ) )
            {
                if ( schema == "dc" && packing == "xml" )
                {
                    entry = new BibTeX::Entry( BibTeX::Entry::etMisc, "SRU" );
                    parseDublinCore( e, entry, FALSE );
                }
                else
                {
                    kdDebug() << "Combination of schema \"" << schema << "\" and packing \"" << packing << "\" is not supported" << endl;
                }
            }
        }

        return entry;
    }

    void SRU::ResultParser::parseDublinCore( const QDomElement & element, BibTeX::Entry * entry, bool isDCX )
    {
// FIXME
//         QStringList authorList;
//
//         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
//         {
//             QDomElement e = n.toElement();
//             if ( !e.isNull() )
//             {
//                 kdDebug() << "e.tagName=" << e.tagName() << endl;
//                 if (( e.tagName() == "dcx" || e.tagName() == "srw_dc:dc" ) && !isDCX )
//                 {
//                     parseDublinCore( e, entry, TRUE );
//                     break;
//                 }
//                 else if ( e.tagName().endsWith( "title" ) )
//                 {
//                     BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftTitle );
//                     if ( field == NULL )
//                     {
//                         field = new BibTeX::EntryField( BibTeX::EntryField::ftTitle );
//                         entry->addField( field );
//                     }
//                     BibTeX::Value *value = new BibTeX::Value();
//                     value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
//                     field->setValue( value );
//                 }
//                 else if ( e.tagName().endsWith( "year" ) )
//                 {
//                     BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftTitle );
//                     if ( field == NULL )
//                     {
//                         field = new BibTeX::EntryField( BibTeX::EntryField::ftYear );
//                         entry->addField( field );
//                     }
//                     BibTeX::Value *value = new BibTeX::Value();
//                     value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
//                     field->setValue( value );
//                 }
//                 else if ( e.tagName().endsWith( "volume" ) )
//                 {
//                     BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftTitle );
//                     if ( field == NULL )
//                     {
//                         field = new BibTeX::EntryField( BibTeX::EntryField::ftVolume );
//                         entry->addField( field );
//                     }
//                     BibTeX::Value *value = new BibTeX::Value();
//                     value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
//                     field->setValue( value );
//                 }
//                 else if ( e.tagName().endsWith( "creator" ) || e.tagName().endsWith( "author" ) )
//                 {
//                     QStringList nameFragments = QStringList::split( QRegExp( ",\\s+" ), e.text() );
//                     if ( nameFragments.count() >= 2 )
//                         authorList.append( nameFragments[ 1 ] + " " + nameFragments[ 0 ] );
//                 }
//                 else if ( e.tagName().endsWith( "publisher" ) )
//                 {
//                     QStringList publisherFragments = QStringList::split( QRegExp( ",\\s+" ), e.text() );
//                     if ( publisherFragments.count() >= 1 )
//                     {
//                         QStringList publisher2Fragments = QStringList::split( QRegExp( "\\s+:\\s+" ), publisherFragments[ 0 ] );
//                         BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftPublisher );
//                         if ( field == NULL )
//                         {
//                             field = new BibTeX::EntryField( BibTeX::EntryField::ftPublisher );
//                             entry->addField( field );
//                         }
//                         BibTeX::Value *value = new BibTeX::Value();
//                         ;
//                         value->add( new BibTeX::ValueItem( publisherFragments.count() >= 2 ? publisher2Fragments[ 1 ] : publisherFragments[ 0 ], TRUE ) );
//                         field->setValue( value );
//                     }
//                 }
//             }
//         }
    }

    SRU::ResultsListViewItem::ResultsListViewItem( QListView * parent, BibTeX::Entry * entry ) : QListViewItem( parent ), m_entry( entry )
    {
// FIXME
        /*        BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftTitle );
                if ( field != NULL && field->value() != NULL )
                    setText( 2, field ->value() ->plainString() );
                field = entry->getField( BibTeX::EntryField::ftAuthor );
                if ( field != NULL && field->value() != NULL )
                {
                    BibTeX::ValuePersons * persons = dynamic_cast<BibTeX::ValuePersons*>( field->value() );
                    if ( persons != NULL )
                    {
                        QValueList<BibTeX::Person*> list = persons->persons();
                        QStringList authors;
                        for ( QValueList<BibTeX::Person*>::ConstIterator it = list.begin();it != list.end(); ++it )
                            authors.append(( *it ) ->text() );
                        setText( 1, authors.join( " and " ) );
                    }
                    else setText( 1, field ->value() ->plainString() );
                }
                field = entry->getField( BibTeX::EntryField::ftYear );
                if ( field != NULL && field->value() != NULL )
                    setText( 0, field ->value() ->plainString() );*/
    }

    SRU::ResultsListViewItem::~ResultsListViewItem()
    {
        if ( m_entry != NULL )
            delete m_entry;
    }

    BibTeX::Entry* SRU::ResultsListViewItem::entry()
    {
        return m_entry;
    }
}
