/***************************************************************************
*   Copyright (C) 2004-2006 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef KBIBTEXWEBQUERYSRU_H
#define KBIBTEXWEBQUERYSRU_H

#include <qlistview.h>

#include <kdialogbase.h>

#include <webquery.h>

class QComboBox;
class QDomElement;
class QLineEdit;
class QSpinBox;
class QPushButton;
class QHttp;
class QTextEdit;

namespace KBibTeX
{
    namespace SRU
    {
        class SRUWizard: public KDialogBase
        {
            Q_OBJECT
        public:
            SRUWizard( const QString &caption, QWidget* parent, const char* name = 0 );
            ~SRUWizard();

            BibTeX::File *result();

        public slots:
            int exec();
            void accept();

        private slots:
            void startSearch();
            void searchRetrieveDone( bool );
            void listViewResultsSelectionChanged( );
            void httpStateChanged( int state );

        private:
            QHttp *http;
            QByteArray incomingData;
            QComboBox *m_comboBoxServer;
            QLineEdit *m_lineEditQuery;
            QSpinBox *m_spinBoxMaxHits;
            QPushButton *m_pushButtonSearch;
            QListView *m_listViewResults;
            QTextEdit *m_sourceWidget;
            BibTeX::File *bibtexFile;

            void setupGUI();
        };

        class ResultParser
        {
        public:
            ResultParser( QListView *listView, const QDomElement& rootElement );
            ~ResultParser();

        private:
            BibTeX::Entry * parseRecord( const QDomElement& element );
            void parseDublinCore( const QDomElement& element, BibTeX::Entry *entry, bool isDCX );
        };

        class ResultsListViewItem: public QListViewItem
        {
        public:
            ResultsListViewItem( QListView * parent, BibTeX::Entry *entry );
            ~ResultsListViewItem();

            BibTeX::Entry* entry();
        private:
            BibTeX::Entry *m_entry;
        };
    }

    /**
     @author Thomas Fischer <fischer@unix-ag.uni-kl.de>
    */
    class WebQuerySRU : public WebQuery
    {
        Q_OBJECT
    public:
        WebQuerySRU( QWidget* parent, const char* name = 0 );
        ~WebQuerySRU();

        BibTeX::File* query( );

        QString title();
        int uniqueId();

    private:
        SRU::SRUWizard *wizard;
    };

}

#endif
